// Fix Point Arithmetic Defines
// -----------------------------------------------
//	Based on ARM article


// Basic Operations: 2 Fixed Args

#ifndef __FIXMATH_H
#define __FIXMATH_H

#define FADD(a,b) ((a)+(b))
#define FSUB(a,b) ((a)-(b))
#define FMUL(a,b) (((a)*(b))>>(8))
#define FDIV(a,b) (((a)<<(8))/(b))

#define FMULQ(a,b, q) (((a)*(b))>>(q))
#define FDIVQ(a,b, q) (((a)<<(q))/(b))

// Basic Operations: 1 Fixed, 1 Int Arg

#define FADDI(a,b,q) ((a)+((b)<<(q)))
#define FSUBI(a,b,q) ((a)-((b)<<(q)))
#define FMULI(a,b) ((a)*(b))
#define FDIVI(a,b) ((a)/(b))

// Q Conversion

#define FCONV(a, q1, q2) (((q2)>(q1)) ? (a)<<((q2)-(q1)) : (a)>>((q1)-(q2)))

// Basic Operations: 2 Fixed Args, each in different q.  Result in third q

#define FADDG(a,b,q1,q2,q3) (FCONV(a,q1,q3)+FCONV(b,q2,q3))
#define FSUBG(a,b,q1,q2,q3) (FCONV(a,q1,q3)-FCONV(b,q2,q3))
#define FMULG(a,b,q1,q2,q3) FCONV((a)*(b), (q1)+(q2), q3)
#define FDIVG(a,b,q1,q2,q3) (FCONV(a, q1, (q2)+(q3))/(b))

// Conversion Functions

#define FIX(d) ((int)( (d)*(double)(1<<(8)) ))
#define FLT(a) ( (double)(a) / (double)(1<<(8)) )
#define WHOLE(d) ( (int) d >> 8 )

#define FIXQ(d,q) ((int)( (d)*(double)(1<<(q)) ))
#define FLTQ(a,q) ( (double)(a) / (double)(1<<(q)) )
#define WHOLEQ(d,q) ( (int) d >> q )

#endif