/////////////////////////////////
// PONG FOR GAME PARK 32
// ph0x (ph0x@freemail.hu)
// http://people.inf.elte.hu/ph0x
/////////////////////////////////

#include <gp32.h>

#include "fixmath.h"

// misc
#define PAD_SIZE		32
#define BALL_SIZE		4

GPDRAWSURFACE surface;

long    i,          // general purpose :)
	key,	// key status
	x1, y1, // player1
	x2, y2, // player2 (computer)
	x, y,	// ball
	dx, dy, // offset
	playing;// game loop flag

__inline void DrawPad(int x, int y, int color) {
	int* buffer=(int*)(surface.ptbuffer+x*GPC_LCD_HEIGHT+y);
	for (i=0; i<PAD_SIZE; i+=4) *(int*)(++buffer)=color;
}

__inline void DrawBall(int x, int y, int color) {
	for (i=0; i<BALL_SIZE; i++) *(unsigned int*)(surface.ptbuffer+(x+i)*GPC_LCD_HEIGHT+y)=color;
}

__inline void DrawNet() {
	int* buffer=(int*)(surface.ptbuffer+160*GPC_LCD_HEIGHT);
	for (i=0; i<GPC_LCD_HEIGHT/2; i+=2) *(unsigned short*)(buffer+i)=(255<<8)+255;
}

int main(void) {

        gpSurfaceSet=(tGpSurfaceSet*)Init();

	surface.buf_w= GPC_LCD_WIDTH;
	surface.buf_h= GPC_LCD_HEIGHT;
	surface.ox=surface.oy=0;
	GpSurfaceCreate(&surface, surface.bufflag);
	gpSurfaceSet(&surface);
	gpLcdEnable();

        while (1) {        // loop forever

                // clear/set bg
                for (i=0; i<(GPC_LCD_WIDTH*GPC_LCD_HEIGHT); i+=4)
                        *(unsigned long*)(surface.ptbuffer+i)=0;

		x1=10; y1=FIX((GPC_LCD_HEIGHT-PAD_SIZE)/2);
		x2=GPC_LCD_WIDTH-10; y2=(GPC_LCD_HEIGHT-PAD_SIZE)/2;
		x=FIX(160); y=FIX(120);
		dx=dy=FIX(0.05);
		playing=TRUE;

                DrawNet();
                DrawPad(x1, WHOLE(y1), COLOR_WHITE);
                DrawPad(x2, y2, COLOR_WHITE);
                DrawBall(WHOLE(x), WHOLE(y), COLOR_WHITE);

                while((gpKeyGet()&0xff) == 0xff);  // wait for keypress

		while (playing) {
			DrawNet();
			DrawPad(x1, WHOLE(y1), COLOR_WHITE);
			DrawPad(x2, y2, COLOR_WHITE);
			DrawBall(WHOLE(x), WHOLE(y), COLOR_WHITE);

			for (i=0; i<100; i++);	// delay
			DrawPad(x2, y2, COLOR_BLACK);
			DrawBall(WHOLE(x), WHOLE(y), COLOR_BLACK);

			if (WHOLE(x)>x2-BALL_SIZE) dx=-dx;
			if (WHOLE(x)==x1) {
				if (y>=y1 && y<=y1+FIX(PAD_SIZE)) dx=-dx; else playing=FALSE;	// hit or miss?
			}
			if (y<FIX(0) || y>FIX(GPC_LCD_HEIGHT-1)) dy=-dy; // bounce off
			x+=dx;
			y+=dy;
			y2=WHOLE(y)-PAD_SIZE/2;
			if (y2<0) y2=0;
			if (y2>GPC_LCD_HEIGHT-PAD_SIZE) y2=GPC_LCD_HEIGHT-PAD_SIZE;	// ball hits pad2

                        key=((~(gpKeyGet()))&0xff);

                        if ((key & GPC_VK_UP) != 0) {
                                if (y1<FIX(GPC_LCD_HEIGHT-PAD_SIZE)) {
                                        DrawPad(x1, WHOLE(y1), COLOR_BLACK);
                                        y1+= FIX(0.05); // move pad up
                                }
                        }
                        if ((key & GPC_VK_DOWN) != 0) {
                                if (y1>FIX(1)) {
                                        DrawPad(x1, WHOLE(y1), COLOR_BLACK);
                                        y1-= FIX(0.05); // move pad down
                                }
                        }
		}

                while((gpKeyGet()&0xff) == 0xff);  // wait for keypress
	}

        return(0);
}
